==> wss4j-policy-2.0.2-sources.jar: org/apache/wss4j/policy/model/UsernameToken.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.wss4j.policy.model;

import org.apache.neethi.Assertion;
import org.apache.neethi.Policy;
import org.apache.wss4j.policy.SPConstants;
import org.w3c.dom.Element;

import javax.xml.namespace.QName;
import java.util.*;

public class UsernameToken extends AbstractToken {

    public enum PasswordType {
        NoPassword,
        HashPassword;

        private static final Map<String, PasswordType> lookup = new HashMap<String, PasswordType>();

        static {
            for (PasswordType u : EnumSet.allOf(PasswordType.class))
                lookup.put(u.name(), u);
        }

        public static PasswordType lookUp(String name) {
            return lookup.get(name);
        }
    }

    public enum UsernameTokenType {
        WssUsernameToken10,
        WssUsernameToken11;

        private static final Map<String, UsernameTokenType> lookup = new HashMap<String, UsernameTokenType>();

        static {
            for (UsernameTokenType u : EnumSet.allOf(UsernameTokenType.class))
                lookup.put(u.name(), u);
        }

        public static UsernameTokenType lookUp(String name) {
            return lookup.get(name);
        }
    }

    private PasswordType passwordType;
    private boolean created;
    private boolean nonce;
    private UsernameTokenType usernameTokenType;

    public UsernameToken(SPConstants.SPVersion version, SPConstants.IncludeTokenType includeTokenType,
                         Element issuer, String issuerName, Element claims, Policy nestedPolicy) {
        super(version, includeTokenType, issuer, issuerName, claims, nestedPolicy);

        parseNestedPolicy(nestedPolicy, this);
    }

    @Override
    public QName getName() {
        return getVersion().getSPConstants().getUsernameToken();
    }

    @Override
    protected AbstractSecurityAssertion cloneAssertion(Policy nestedPolicy) {
        return new UsernameToken(getVersion(), getIncludeTokenType(), getIssuer(), getIssuerName(), getClaims(), nestedPolicy);
    }

    protected void parseNestedPolicy(Policy nestedPolicy, UsernameToken usernameToken) {
        Iterator<List<Assertion>> alternatives = nestedPolicy.getAlternatives();
        //we just process the first alternative
        //this means that if we have a compact policy only the first alternative is visible
        //in contrary to a normalized policy where just one alternative exists
        if (alternatives.hasNext()) {
            List<Assertion> assertions = alternatives.next();
            for (int i = 0; i < assertions.size(); i++) {
                Assertion assertion = assertions.get(i);
                String assertionName = assertion.getName().getLocalPart();
                String assertionNamespace = assertion.getName().getNamespaceURI();
                PasswordType passwordType = PasswordType.lookUp(assertionName);
                if (passwordType != null) {
                    if (usernameToken.getPasswordType() != null) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setPasswordType(passwordType);
                    continue;
                }
                if (getVersion().getSPConstants().getCreated().getLocalPart().equals(assertionName)
                        && getVersion().getSPConstants().getCreated().getNamespaceURI().equals(assertionNamespace)) {
                    if (usernameToken.isCreated()) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setCreated(true);
                    continue;
                }
                if (getVersion().getSPConstants().getNonce().getLocalPart().equals(assertionName)
                        && getVersion().getSPConstants().getNonce().getNamespaceURI().equals(assertionNamespace)) {
                    if (usernameToken.isNonce()) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setNonce(true);
                    continue;
                }
                DerivedKeys derivedKeys = DerivedKeys.lookUp(assertionName);
                if (derivedKeys != null) {
                    if (usernameToken.getDerivedKeys() != null) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setDerivedKeys(derivedKeys);
                    continue;
                }
                UsernameTokenType usernameTokenType = UsernameTokenType.lookUp(assertionName);
                if (usernameTokenType != null) {
                    if (usernameToken.getUsernameTokenType() != null) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setUsernameTokenType(usernameTokenType);
                    continue;
                }
            }
        }
    }

    public PasswordType getPasswordType() {
        return passwordType;
    }

    protected void setPasswordType(PasswordType passwordType) {
        this.passwordType = passwordType;
    }

    public boolean isCreated() {
        return created;
    }

    protected void setCreated(boolean created) {
        this.created = created;
    }

    public boolean isNonce() {
        return nonce;
    }

    protected void setNonce(boolean nonce) {
        this.nonce = nonce;
    }

    public UsernameTokenType getUsernameTokenType() {
        return usernameTokenType;
    }

    protected void setUsernameTokenType(UsernameTokenType usernameTokenType) {
        this.usernameTokenType = usernameTokenType;
    }
}

==> wss4j-policy-2.1.3-sources.jar: org/apache/wss4j/policy/model/UsernameToken.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.wss4j.policy.model;

import org.apache.neethi.Assertion;
import org.apache.neethi.Policy;
import org.apache.wss4j.policy.SPConstants;
import org.w3c.dom.Element;

import javax.xml.namespace.QName;
import java.util.*;

public class UsernameToken extends AbstractToken {

    public enum PasswordType {
        NoPassword,
        HashPassword;

        private static final Map<String, PasswordType> lookup = new HashMap<>();

        static {
            for (PasswordType u : EnumSet.allOf(PasswordType.class))
                lookup.put(u.name(), u);
        }

        public static PasswordType lookUp(String name) {
            return lookup.get(name);
        }
    }

    public enum UsernameTokenType {
        WssUsernameToken10,
        WssUsernameToken11;

        private static final Map<String, UsernameTokenType> lookup = new HashMap<>();

        static {
            for (UsernameTokenType u : EnumSet.allOf(UsernameTokenType.class))
                lookup.put(u.name(), u);
        }

        public static UsernameTokenType lookUp(String name) {
            return lookup.get(name);
        }
    }

    private PasswordType passwordType;
    private boolean created;
    private boolean nonce;
    private UsernameTokenType usernameTokenType;

    public UsernameToken(SPConstants.SPVersion version, SPConstants.IncludeTokenType includeTokenType,
                         Element issuer, String issuerName, Element claims, Policy nestedPolicy) {
        super(version, includeTokenType, issuer, issuerName, claims, nestedPolicy);

        parseNestedPolicy(nestedPolicy, this);
    }

    @Override
    public QName getName() {
        return getVersion().getSPConstants().getUsernameToken();
    }

    @Override
    protected AbstractSecurityAssertion cloneAssertion(Policy nestedPolicy) {
        return new UsernameToken(getVersion(), getIncludeTokenType(), getIssuer(), getIssuerName(), getClaims(), nestedPolicy);
    }

    protected void parseNestedPolicy(Policy nestedPolicy, UsernameToken usernameToken) {
        Iterator<List<Assertion>> alternatives = nestedPolicy.getAlternatives();
        //we just process the first alternative
        //this means that if we have a compact policy only the first alternative is visible
        //in contrary to a normalized policy where just one alternative exists
        if (alternatives.hasNext()) {
            List<Assertion> assertions = alternatives.next();
            for (int i = 0; i < assertions.size(); i++) {
                Assertion assertion = assertions.get(i);
                String assertionName = assertion.getName().getLocalPart();
                String assertionNamespace = assertion.getName().getNamespaceURI();
                PasswordType passwordType = PasswordType.lookUp(assertionName);
                if (passwordType != null) {
                    if (usernameToken.getPasswordType() != null) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setPasswordType(passwordType);
                    continue;
                }
                if (getVersion().getSPConstants().getCreated().getLocalPart().equals(assertionName)
                        && getVersion().getSPConstants().getCreated().getNamespaceURI().equals(assertionNamespace)) {
                    if (usernameToken.isCreated()) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setCreated(true);
                    continue;
                }
                if (getVersion().getSPConstants().getNonce().getLocalPart().equals(assertionName)
                        && getVersion().getSPConstants().getNonce().getNamespaceURI().equals(assertionNamespace)) {
                    if (usernameToken.isNonce()) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setNonce(true);
                    continue;
                }
                DerivedKeys derivedKeys = DerivedKeys.lookUp(assertionName);
                if (derivedKeys != null) {
                    if (usernameToken.getDerivedKeys() != null) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setDerivedKeys(derivedKeys);
                    continue;
                }
                UsernameTokenType usernameTokenType = UsernameTokenType.lookUp(assertionName);
                if (usernameTokenType != null) {
                    if (usernameToken.getUsernameTokenType() != null) {
                        throw new IllegalArgumentException(SPConstants.ERR_INVALID_POLICY);
                    }
                    usernameToken.setUsernameTokenType(usernameTokenType);
                    continue;
                }
            }
        }
    }

    public PasswordType getPasswordType() {
        return passwordType;
    }

    protected void setPasswordType(PasswordType passwordType) {
        this.passwordType = passwordType;
    }

    public boolean isCreated() {
        return created;
    }

    protected void setCreated(boolean created) {
        this.created = created;
    }

    public boolean isNonce() {
        return nonce;
    }

    protected void setNonce(boolean nonce) {
        this.nonce = nonce;
    }

    public UsernameTokenType getUsernameTokenType() {
        return usernameTokenType;
    }

    protected void setUsernameTokenType(UsernameTokenType usernameTokenType) {
        this.usernameTokenType = usernameTokenType;
    }
}
